%%*******************************************************************
%% HKMrhsfun: compute the right-hand side vector of the 
%%            Schur complement equation for the  HKM direction. 
%% 
%% SDPT3: version 3.0 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 2 Feb 01
%%*******************************************************************

  function [rhs,EinvRc,hRd] = HKMrhsfun(blk,Avec,par,X,Z,rp,Rd,sigmu,hRd,dX,dZ)

    global Zinv gamz dd ee

    m = length(rp);   
    if (nargin > 8); 
       corrector = 1; 
    else; 
       corrector = 0; 
       hRd = zeros(m,1); 
    end       
    hEinvRc = zeros(m,1); 
    exist_free_vars = 0; rhsfree = []; 
%%
    for p = 1:size(blk,1)
       pblk = blk(p,:); 
       n = sum(pblk{2}); 
       if strcmp(pblk{1},'l')
          if corrector 
             Rq = dX{p}.*dZ{p}; 
          else
             Rq = sparse(n,1); 
             tmp  = dd{p}.*Rd{p};
             tmp2 = (tmp'*Avec{p})';
             hRd = hRd + tmp2;
          end
          EinvRc{p} = sigmu./Z{p}-X{p} -Rq./Z{p};
          tmp2 = (EinvRc{p}'*Avec{p})';  
          hEinvRc = hEinvRc + tmp2;
       elseif strcmp(pblk{1},'q') 
          if corrector
  	     ff{p} = qops(pblk,1./gamz{p},Z{p},3);
             hdx = qops(pblk,gamz{p},ff{p},5,dX{p}); 
             hdz = qops(pblk,gamz{p},ff{p},6,dZ{p}); 
             hdxdz = Arrow(pblk,hdx,hdz);
             Rq = qops(pblk,gamz{p},ff{p},6,hdxdz); 
          else
             Rq  = sparse(n,1); 
             tmp = dd{p}.*Rd{p} ...
                   + qops(pblk,qops(pblk,Rd{p},ee{p},1),X{p},3) ...
                   + qops(pblk,qops(pblk,Rd{p},X{p},1),ee{p},3); 
             tmp2 = (tmp'*Avec{p})';
             hRd = hRd + tmp2;
          end
          EinvRc{p} = sigmu*ee{p}-X{p} -Rq;
          tmp2 = (EinvRc{p}'*Avec{p})';         
          hEinvRc = hEinvRc + tmp2;
       elseif strcmp(pblk{1},'s') 
          if corrector  
             Rq = Prod3(pblk,dX{p},dZ{p},Zinv{p},0); 
	     Rq = 0.5*(Rq+Rq'); 
          else
             Rq = sparse(n,n); 
             if (norm(Rd{p},'fro') > 1e-13) 
                tmp = Prod3(pblk,X{p},Rd{p},Zinv{p},0,par.nzlistAy{p}); 
                tmp = 0.5*(tmp+tmp'); 
                tmp2(par.permA(p,:),1) = mexinprod(blk,Avec,svec(pblk,tmp),m,p);
                hRd = hRd + tmp2;
             end
          end 
          tmp = sigmu*Zinv{p}-X{p}- Rq; 
          EinvRc{p} = tmp;
          tmp2(par.permA(p,:),1) = mexinprod(blk,Avec,svec(pblk,tmp),m,p);
          hEinvRc = hEinvRc + tmp2;
       elseif strcmp(pblk{1},'u') 
          exist_free_vars = 1; 
          rhsfree = [rhsfree; Rd{p}]; 
       end
    end
%%
    rhs = rp + hRd - hEinvRc; 
    if exist_free_vars
       rhs = [rhs; rhsfree];  
    end
%%*******************************************************************
