/***********************************************************************
* mexqprodfun.c : C mex file 
*
*   jj = mexqprod(blk); 
* 
*   Input: blk   = [n1, n2, ..., nk]
*
*   Output: [1 1...1  2 2 ... 2 .... k k ...k]'    
*            n1       n2             nk
*
* SDPT3: version 3.0
* Copyright (c) 1997 by
* K.C. Toh, M.J. Todd, R.H. Tutuncu
* Last Modified: 2 Feb 01   
***********************************************************************/

#include <math.h>
#include <mex.h>

/**********************************************************
* 
***********************************************************/
void mexFunction(
      int nlhs,   mxArray  *plhs[], 
      int nrhs,   const mxArray  *prhs[] )

{    mxArray  *blk_cell_pr;
     double   *blksize, *jj; 

     int   subs[2];
     int   nsubs=2; 
     int   index, k, l, blkdim, numblk, cols, idx;

/* CHECK FOR PROPER NUMBER OF ARGUMENTS */

   if (nrhs !=1){
      mexErrMsgTxt("mexqprodfun: requires 1 input arguments."); }
   if (nlhs > 1){ 
      mexErrMsgTxt("mexqprodfun: requires 1 output argument."); }

/* CHECK THE DIMENSIONS */

    if (mxGetM(prhs[0]) > 1) { 
       mexErrMsgTxt("mexqprodfun: blk can have only 1 row."); }   
    if (!mxIsCell(prhs[0])) {
        mexErrMsgTxt("mexqprodfun: blk must be a cell array."); }
    subs[0]=0; subs[1]=1; 
    index = mxCalcSingleSubscript(prhs[0],nsubs,subs); 
    blk_cell_pr = mxGetCell(prhs[0],index);
    numblk = mxGetN(blk_cell_pr); 
    blksize = mxGetPr(blk_cell_pr);
    cols = 0; 
    for (k=0; k<numblk; k++) { 
        cols = cols + (int)blksize[k]; 
    } 
    plhs[0] = mxCreateDoubleMatrix(cols,1,mxREAL); 
    jj = mxGetPr(plhs[0]);    
    idx = 0; 
    for (k=1; k<=numblk; k++) { 
       blkdim = (int)blksize[k-1]; 
       for (l=0; l<blkdim; l++) { jj[idx] = k; idx++; }
    }
    return;
 }
/**********************************************************/

