%%*******************************************************************
%% NTrhsfun: compute the right-hand side vector of the 
%%           Schur complement equation for the NT direction. 
%% 
%% SDPT3: version 3.0 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 2 Feb 01
%%*******************************************************************

    function [rhs,EinvRc,hRd] = NTrhsfun(blk,Avec,par,X,Z,rp,Rd,sigmu,hRd,dX,dZ); 
    
    global spdensity 
    global W G sv gamx gamz ff dd ee 
 
    m = length(rp);   
    if (nargin > 8) 
       corrector = 1; 
    else 
       corrector = 0; 
       hRd = zeros(m,1); 
    end       
    hEinvRc = zeros(m,1); 
    exist_free_vars = 0; rhsfree = []; 
%%
    for p = 1:size(blk,1)
        pblk = blk(p,:); 
        n = sum(pblk{2});  numblk = length(pblk{2});  
        if strcmp(pblk{1},'l')
           if corrector 
              Rq = dX{p}.*dZ{p}; 
           else
              Rq = sparse(n,1); 
              tmp  = dd{p}.*Rd{p};
              tmp2 = (tmp'*Avec{p})';
              hRd = hRd + tmp2;
           end
           EinvRc{p} = sigmu./Z{p}-X{p} -Rq./Z{p};
           tmp2 = (EinvRc{p}'*Avec{p})';  
           hEinvRc = hEinvRc + tmp2;
	elseif strcmp(pblk{1},'q') 
           w = sqrt(gamz{p}./gamx{p}); 
           if corrector
              hdx = qops(pblk,w,ff{p},5,dX{p}); 
              hdz = qops(pblk,w,ff{p},6,dZ{p}); 
              hdxdz = Arrow(pblk,hdx,hdz);
              vv = qops(pblk,w,ff{p},5,X{p}); 
              Vihdxdz = Arrow(pblk,vv,hdxdz,1); 
              Rq = qops(pblk,w,ff{p},6,Vihdxdz); 
           else
              Rq = sparse(n,1); 
              tmp  = dd{p}.*Rd{p} + qops(pblk,qops(pblk,Rd{p},ee{p},1),ee{p},3);
              tmp2 = (tmp'*Avec{p})';
              hRd = hRd + tmp2;
           end
           EinvRc{p} = qops(pblk,-sigmu./(gamz{p}.*gamz{p}),Z{p},4)-X{p}-Rq;
           tmp2 = (EinvRc{p}'*Avec{p})';         
           hEinvRc = hEinvRc + tmp2;
        elseif strcmp(pblk{1},'s') 
           n2 = pblk{2}.*(pblk{2}+1)/2; 
           if corrector  
              hdZ = Prod3(pblk,G{p},dZ{p},G{p}',1); 
              hdX = spdiags(-sv{p},0,n,n)-hdZ;          
              tmp = Prod2(pblk,hdX,hdZ,0);  clear hdX hdZ; 
              tmp = 0.5*(tmp+tmp');
              if (numblk == 1) 
                 d = sv{p};
       	         e = ones(pblk{2},1); 
     	         Rq = 2*tmp./(d*e'+e*d'); clear tmp; 
                 if (nnz(Rq) <= spdensity*n2); Rq = sparse(Rq); end
              else
                 Rq = sparse(n,n);
                 s = [0 cumsum(pblk{2})]; 
                 for i = 1:numblk
                     pos = [s(i)+1 : s(i+1)]; 
                     d = sv{p}(pos); e = ones(length(pos),1); 
                     Rq(pos,pos) = 2*tmp(pos,pos)./(d*e' + e*d'); 
                 end
              end
           else
              Rq = sparse(n,n); 
              if (norm(Rd{p},'fro') > 1e-13) 
                 tmp = Prod3(pblk,W{p},Rd{p},W{p},1,par.nzlistAy{p});
                 tmp2(par.permA(p,:),1) = mexinprod(blk,Avec,svec(pblk,tmp),m,p);
                 hRd = hRd + tmp2;
              end
           end 
           tmp = spdiags(sigmu./sv{p} -sv{p},0,n,n);
           tmp = Prod3(pblk,G{p}',tmp-Rq,G{p},1);
           EinvRc{p} = tmp;
           tmp2(par.permA(p,:),1) = mexinprod(blk,Avec,svec(pblk,tmp),m,p);
           hEinvRc = hEinvRc + tmp2;  
        elseif strcmp(pblk{1},'u') 
           exist_free_vars = 1; 
           rhsfree = [rhsfree; Rd{p}]; 
        end 
    end
%% 
    rhs = rp + hRd - hEinvRc; 
    if exist_free_vars
       rhs = [rhs; rhsfree];  
    end
%%*******************************************************************
