function y = minus(X,Y)
%MINUS (overloaded)

% Author Johan Lfberg
% $Id: minus.m,v 1.7 2005/01/10 09:14:44 johanl Exp $

X_is_spdvar = isa(X,'sdpvar');
Y_is_spdvar = isa(Y,'sdpvar');

switch 2*X_is_spdvar+Y_is_spdvar
    case 1
        if isempty(X)
            y = [];
            return
        end

        y = Y;
        n_Y = Y.n;
        m_Y = Y.m;
        [n_X,m_X] = size(X);
        x_isscalar = (n_X*m_X==1);
        y_isscalar = (n_Y*m_Y==1);
        any_scalar = x_isscalar | y_isscalar;

        if any_scalar | ([n_Y m_Y]==[n_X m_X])
            if y_isscalar
                y.basis = repmat(y.basis,n_X*m_X,1);
                y.n = n_X;
                y.m = m_X;
            end
            y.basis = -y.basis;
            y.basis(:,1) = y.basis(:,1)+X(:);
        else
            error('Matrix dimensions must agree.');
        end
        if nnz(X)~=0
          %  y.unaltered = 0;
        end
            
    case 2

        %	if X.typeflag~=0
        %		error('Relational objects cannot be manipulated')
        %	end
        if isempty(Y)
            y=[];
            return
        end
        y = X;
        n_X = X.n;
        m_X = X.m;
        [n_Y,m_Y] = size(Y);
        x_isscalar = (n_X*m_X==1);
        y_isscalar = (n_Y*m_Y==1);
        any_scalar = x_isscalar | y_isscalar;

        % Silly hack
        % Taking X-scalar(0) takes unnecessary time
        % and is used in most definitions of LMIs
        if (y_isscalar & (Y==0))
            return
        end

        if any_scalar | ([n_Y m_Y]==[n_X m_X])
            if x_isscalar
                y.basis = repmat(y.basis,n_Y*m_Y,1);
                y.n = n_Y;
                y.m = m_Y;
            end
            y.basis(:,1) = y.basis(:,1)-Y(:);
        else
            error('Matrix dimensions must agree.');
        end
        if nnz(Y)~=0
           % y.unaltered = 0;
        end

    case 3

        %	if (X.typeflag~=0) | (Y.typeflag~=0)
        %		error('Relational objects cannot be manipulated')
        %	end

        n_X = X.n;
        m_X = X.m;
        n_Y = Y.n;
        m_Y = Y.m;
        x_isscalar = (n_X*m_X==1);
        y_isscalar = (n_Y*m_Y==1);
        any_scalar = x_isscalar | y_isscalar;

        if (~((n_X==n_Y) & (m_X==m_Y))) & ~any_scalar
            error('Matrix dimensions must agree.')
        end

        all_lmi_variables = uniquestripped([X.lmi_variables Y.lmi_variables]);
        y = X;
        y.lmi_variables = all_lmi_variables;

        in_X = find(ismembc(all_lmi_variables,X.lmi_variables));
        in_Y = find(ismembc(all_lmi_variables,Y.lmi_variables));

%         % Preallocate good for large problems
%         if 0
%             [ix,jx,sx] = find(X.basis);
%             [iy,jy,sy] = find(Y.basis);
%             mapX = [1 1+in_X];
%             mapY = [1 1+in_Y];
%             jx = mapX(jx);
%             jy = mapY(jy);
%             basis_X = sparse(ix,jx,sx,n_X*m_X,1+length(all_lmi_variables));
%             basis_Y = sparse(iy,jy,sy,n_Y*m_Y,1+length(all_lmi_variables));
%         else
%             basis_X = spalloc(n_X*m_X,1+length(all_lmi_variables),nnz(X.basis));
%             basis_Y = spalloc(n_Y*m_Y,1+length(all_lmi_variables),nnz(Y.basis));
%             basis_X(:,[1 1+in_X])=X.basis;
%             basis_Y(:,[1 1+in_Y])=Y.basis;
%         end

        if 1
           [ix,jx,sx] = find(X.basis);
           [iy,jy,sy] = find(Y.basis);
           mapX = [1 1+in_X];
           mapY = [1 1+in_Y];
           basis_X = sparse(ix,mapX(jx),sx,n_X*m_X,1+length(all_lmi_variables));
           basis_Y = sparse(iy,mapY(jy),sy,n_Y*m_Y,1+length(all_lmi_variables));
       else
           % MATLAB sparse fails on this for huge problems at a certain size
           basis_X = spalloc(n_X*m_X,1+length(all_lmi_variables),nnz(X.basis));
           basis_Y = spalloc(n_Y*m_Y,1+length(all_lmi_variables),nnz(Y.basis));
           basis_X(:,[1 1+in_X])=X.basis;
           basis_Y(:,[1 1+in_Y])=Y.basis;
       end

        
        % Fix addition of matrix+scalar
        if n_X*m_X<n_Y*m_Y
            y.n = n_Y;
            y.m = m_Y;
            basis_X = repmat(basis_X,n_Y*m_Y,1);
        end
        if n_Y*m_Y<n_X*m_X
            y.n = n_X;
            y.m = m_X;
            basis_Y = repmat(basis_Y,n_X*m_X,1);
        end

        % OK, solution is...
        y.basis = basis_X-basis_Y;
        y = clean(y);

    otherwise
end

% Update info on KYP objects
if isa(X,'sdpvar') & X.typeflag==9 &  isa(Y,'sdpvar') & Y.typeflag==9
    error('Substraction of KYP objects currently not supported')
end
if isa(Y,'sdpvar') & Y.typeflag==9
    y.extra.M = -Y.extra.M+X;
    y.extra.negated = ~Y.extra.negated;
    return
end
if isa(X,'sdpvar') & X.typeflag==9
    y.extra.M = y.extra.M-Y;
    return
end



