function setsdpvar(X,value,ls)
%SETSDPVAR Assigns a numerical value to an sdpvar
%
% setsdpvar(X,value)   Tries to set the free variables in X so that  
%                      double(X)=value. Notice that other variables
%                      sharing the same free variables will be affected.
%                      If the assignment is infeasible, an error message
%                      will be issued.
%
% setsdpvar(X,value,1) Least square assignment.

% Author Johan Lfberg 
% $Id: setsdpvar.m,v 1.2 2004/07/01 11:17:13 johanl Exp $  

if nargin<3
    ls = 0;
end

if ~isequal(size(X),size(value))
  error('Both arguments must have same size') 
end
if ~isa(X,'sdpvar')
  error('First arguments must be an sdpvar object') 
end

x_lmi_variables = X.lmi_variables;
b = value(:)-X.basis(:,1);
A = X.basis(:,2:end);
feas_var = A\b;

if ~ls
    if norm(A*feas_var-b)>sqrt(eps)
        error('Inconsistent assignment')
    end
end

sol = sdpvar('getSolution');
keep_these = find(~ismember(sol.variables,x_lmi_variables));
sol.optvar = [sol.optvar(keep_these);feas_var(:)];
sol.variables = [sol.variables(keep_these);x_lmi_variables(:)];
sdpvar('setSolution',sol);



