function YESNO = is(F,property)
%IS   Check property of constraint.
%   d = IS(x,property) returns 1 if 'property' holds
%
%   Properties possible to test are: 'elementwise', 'sdp', 'socc',
%   'lmi', 'linear', 'kyp'

% Author Johan Lfberg 
% $Id: is.m,v 1.7 2005/01/05 10:05:03 johanl Exp $   

if isempty(F.clauses)
    YESNO = 0;
else
    for i = 1:length(F.clauses)
        Fi = F.clauses{i}{1};
        switch property
            case 'equality'
                YESNO(i,1) = (Fi.type==3);
            case {'element-wise','elementwise'}
                YESNO(i,1) = (Fi.type==2);
            case 'socc'
                YESNO(i,1) = (Fi.type==4);
            case 'sdp'
                YESNO(i,1) = Fi.type==1;
            case 'lmi'
                YESNO(i,1) = full((((Fi.type==1) | (Fi.type==9)) | ((Fi.type==2) & (prod(size(Fi.data))==1))) & (islinear(Fi.data)));
            case 'linear'
                YESNO(i,1) = islinear(Fi.data);
            case 'kyp'
                YESNO(i,1) = (Fi.type==9);
            case 'sos'
                YESNO(i,1) = (Fi.type==11);
            case 'eig'
                YESNO(i,1) = (Fi.type==10);
            case 'sigmonial'
                monomtable = yalmip('monomtable');
                monomtable = monomtable(getvariables(Fi.data),:);
                YESNO(i,1) = any(find(any(0>monomtable,2) | any(monomtable-fix(monomtable),2)));
            case 'binary'
                YESNO = Fi.type ==  8;
            case 'integer'
                YESNO = Fi.type ==  7;
            case 'logic'
                YESNO(i,1) = Fi.type == 12;
            case 'lowrank'
                YESNO(i,1) = Fi.type == 14;
            otherwise
                YESNO = error('Huh?');
        end
    end
    %  end
end

