function [problem,integer_variables,binary_variables,parametric_variables,quad_info] = catsdp(F,G,h,relax)
%catsdp          Internal function: tries to determine the type of
%optimization problem

% Author Johan Lfberg 
% $Id: newcatsdp.m,v 1.2 2004/12/21 21:57:09 johanl Exp $

Counter = size(F.clauses,2);
Ftype = zeros(Counter,1);

real_data = 1;
int_data  = 0;
bin_data  = 0;
par_data  = 0;

poly_constraint = 0;
bilin_constraint = 0;
sigm_constraint = 0;

integer_variables = [];
binary_variables = [];
parametric_variables = [];
kyp_prob  = 0;

% ***********************************************
% Setup an empty problem definition
% ***********************************************
problem.objective.linear = 0;
problem.objective.quadratic.convex = 0;
problem.objective.quadratic.nonconvex = 0;
problem.objective.polynomial = 0;
problem.objective.maxdet = 0;
problem.objective.sigmonial = 0;

problem.constraint.equalities.linear     = 0;
problem.constraint.equalities.quadratic  = 0;
problem.constraint.equalities.polynomial = 0;
problem.constraint.equalities.sigmonial  = 0;

problem.constraint.inequalities.elementwise.linear = 0;
problem.constraint.inequalities.elementwise.quadratic.convex = 0;
problem.constraint.inequalities.elementwise.quadratic.nonconvex = 0;
problem.constraint.inequalities.elementwise.sigmonial = 0;
problem.constraint.inequalities.elementwise.polynomial = 0;

problem.constraint.inequalities.semidefinite.linear = 0;
problem.constraint.inequalities.semidefinite.quadratic = 0;
problem.constraint.inequalities.semidefinite.polynomial = 0;
problem.constraint.inequalities.semidefinite.sigmonial = 0;

problem.constraint.inequalities.secondordercone = [];
problem.constraint.inequalities.rotatedsecondordercone = [];

problem.constraint.integer = [];
problem.constraint.binary = [];

problem.complex = [];
problem.parametric = [];

% ********************************************************
% Make a list of all globally available discrete variables
% ********************************************************
integer_variables = yalmip('intvariables');
binary_variables  = yalmip('binvariables');
for i = 1:Counter
    if F.clauses{i}{1}.type==7
        integer_variables = union(integer_variables,getvariables(F.clauses{i}{1}.data));
    elseif F.clauses{i}{1}.type==8
        binary_variables = union(binary_variables,getvariables(F.clauses{i}{1}.data));        
    elseif F.clauses{i}{1}.type==13
        parametric_variables = union(parametric_variables,getvariables(F.clauses{i}{1}.data));        
    end   
end


% ********************************************************
% Make a list of all globally available nonlinear variables
% ********************************************************
monomtable = yalmip('monomtable');

temp = (sum(abs(monomtable),2)==1) & (any(monomtable==1,2));
linear_variables = find(temp);
nonlinear_variables = find(~temp);
sigmonial_variables = find(any(0>monomtable,2) | any(monomtable-fix(monomtable),2));

%discrete_data = ~isempty(integer_variables) | ~isempty(binary_variables);
allvars = getvariables(F);
%nononlinear = isempty(find(ismembc(nonlinear_variables,allvars)));

any_discrete_variables = ~isempty(integer_variables) | ~isempty(binary_variables);
any_nonlinear_variables = ~isempty(find(ismembc(nonlinear_variables,allvars)));

for i = 1:Counter

    real_data = real_data & isreal(F.clauses{i}{1}.data); %Complex

    if any_discrete_variables
        int_data = int_data | any(ismember(getvariables(F.clauses{i}{1}.data),integer_variables));
        bin_data = bin_data | any(ismember(getvariables(F.clauses{i}{1}.data),binary_variables));
        par_data = par_data | any(ismember(getvariables(F.clauses{i}{1}.data),parametric_variables));
    end

    if ~any_nonlinear_variables % No nonlinearly parameterized constraints

        switch F.clauses{i}{1}.type
            case 1
                problem.constraint.inequalities.semidefinite.linear = 1;
            case 2
                problem.constraint.inequalities.elementwise.linear = 1;
            case 3
                problem.constraint.equalities.linear = 1;
            case 4
                problem.constraint.inequalities.secondordercone = 1;
            case 5
                problem.constraint.inequalities.rotatedsecondordercone = 1;
            otherwise
        end
    else
        vars = getvariables(F.clauses{i}{1}.data);
        usednonlins = find(ismembc(nonlinear_variables,vars));
        if ~isempty(usednonlins)
            usedsigmonials = find(ismember(sigmonial_variables,vars));
            if ~isempty(usedsigmonials)
                switch F.clauses{i}{1}.type
                    case 1
                        problem.constraint.inequalities.semidefinite.sigmonial = 1;
                    case 2
                        problem.constraint.inequalities.elementwise.sigmonial = 1;
                    case 3
                        problem.constraint.equalities.sigmonial = 1;
                    case 4
                        %   problem.constraint.inequalities.secondordercone = 1;
                    case 5
                        %   problem.constraint.inequalities.rotatedsecondordercone = 1;
                    otherwise
                        error('Report bug in problem classification (sigmonial constraint)');
                end
            else
                deg = degree(F.clauses{i}{1}.data);
                switch deg                                    
                        
                    case 2

                        switch F.clauses{i}{1}.type
                            case 1
                                problem.constraint.inequalities.semidefinite.quadratic = 1;
                            case 2
                                problem.constraint.inequalities.elementwise.quadratic = 1;
                            case 3
                                problem.constraint.equalities.quadratic = 1;
                            case 4
                                %   problem.constraint.inequalities.secondordercone = 1;
                            case 5
                                %   problem.constraint.inequalities.rotatedsecondordercone = 1;
                            otherwise
                                error('Report bug in problem classification (sigmonial constraint)');
                        end                        
                    otherwise
                        switch F.clauses{i}{1}.type
                            case 1
                                problem.constraint.inequalities.semidefinite.polynomial = 1;
                            case 2
                                problem.constraint.inequalities.elementwise.polynmial = 1;
                            case 3
                                problem.constraint.equalities.polynomial = 1;
                            case 4
                                %   problem.constraint.inequalities.secondordercone = 1;
                            case 5
                                %   problem.constraint.inequalities.rotatedsecondordercone = 1;
                            otherwise
                                error('Report bug in problem classification (sigmonial constraint)');
                        end

                end
                %bilin_constraint = bilin_constraint | (deg==2);
                %poly_constraint = poly_constraint | (deg>2);
            end
        else
            switch F.clauses{i}{1}.type
                            case 1
                                problem.constraint.inequalities.semidefinite.linear = 1;
                            case 2
                                problem.constraint.inequalities.elementwise.linear = 1;
                            case 3
                                problem.constraint.equalities.linear = 1;
                            case 4
                                %   problem.constraint.inequalities.secondordercone = 1;
                            case 5
                                %   problem.constraint.inequalities.rotatedsecondordercone = 1;
                            otherwise
                                error('Report bug in problem classification (sigmonial constraint)');
                        end     
        end
    end
end

if int_data    
    problem.constraint.integer = 1;
end
if bin_data
    problem.constraint.binary = 1;
end
if ~real_data
    problem.complex = 1;
end
    
% for i = 1:Counter
%     Ftype(i) = F.clauses{i}{1}.type;
%     real_data = real_data & isreal(F.clauses{i}{1}.data); %Complex
%     if discrete_data
%         this_integer =  any(ismember(getvariables(F.clauses{i}{1}.data),integer_variables));
%         this_binary = any(ismember(getvariables(F.clauses{i}{1}.data),binary_variables));
%         this_parametric = any(ismember(getvariables(F.clauses{i}{1}.data),parametric_variables));
%         int_data = int_data | any(ismember(getvariables(F.clauses{i}{1}.data),integer_variables));
%         bin_data = bin_data | any(ismember(getvariables(F.clauses{i}{1}.data),binary_variables));
%         par_data = par_data | any(ismember(getvariables(F.clauses{i}{1}.data),parametric_variables));
%     else
%         this_integer = [];
%         this_binary = [];
%         this_parametric = [];
%     end
%     if ~nononlinear
%         vars = getvariables(F.clauses{i}{1}.data);      
%             usednonlins = find(ismembc(nonlinear_variables,vars));
%             if ~isempty(usednonlins)
%                 usedsigmonials = find(ismember(sigmonial_variables,vars));
%                 if ~isempty(usedsigmonials)
%                     sigm_constraint = 1;
%                     poly_constraint = 1;
%                 else
%                     deg = degree(F.clauses{i}{1}.data);
%                     bilin_constraint = bilin_constraint | (deg==2);
%                     poly_constraint = poly_constraint | (deg>2);
%                 end
%             end     
%     end
% end

if relax
    problem.constraint.integer = 0;
    problem.constraint.binary = 0;
    problem.constraint.equalities.quadratic  = 0;
    problem.constraint.equalities.polynomial = 0;
    problem.constraint.equalities.sigmonial  = 0;
    problem.constraint.inequalities.elementwise.quadratic.convex = [];
    problem.constraint.inequalities.elementwise.quadratic.nonconvex = [];
    problem.constraint.inequalities.elementwise.sigmonial   = [];
    problem.constraint.inequalities.elementwise.polynomial  = [];    
    problem.constraint.inequalities.semidefinite.quadratic  = [];
    problem.constraint.inequalities.semidefinite.polynomial = [];
    problem.constraint.inequalities.semidefinite.sigmonial  = [];
    int_data  = 0;
    bin_data  = 0;
    poly_constraint = 0;
    bilin_constraint = 0;
    sigm_constraint = 0;
    integer_variables = [];
    binary_variables = [];
end

quad_info = [];
if (~isempty(h)) & ~is(h,'linear') &~relax

    if any(ismember(getvariables(h),sigmonial_variables))

        problem.objective.sigmonial = 1;

    else
        [Q,c,f,x,info] = quaddecomp(h);
        if info==0
            [R,p]=chol(Q);
            if p~=0
                if min(eig(Q))>=-1e-10
                    p=0;
                    [U,S,V]=svd(Q);
                    i = find(diag(S)>1e-10);
                    R = sqrt(S(1:max(i),1:max(i)))*V(:,1:max(i))';
                end
            end
            if p==0
                problem.objective.quadratic.convex = 1;
            else
                problem.objective.quadratic.convex = 1;
            end
            quad_info.Q = Q;
            quad_info.c = c;
            quad_info.f = f;
            quad_info.x = x;
            quad_info.R = R;
            quad_info.p = p;
        else
            problem.objective.polynomial = 1;
        end
    end
else   
    problem.objective.linear = 1;
end