function F = projection(F,x,method)
% projection  Projects polytopic set object (Requires the Multi-parametric Toolbox).
%
% Fproj     = projection(F,x)
%
% F      : Polytopic set object
% x      : Variables to project on
% method : See HELP PROJECTION

% Author Johan Lfberg
% $Id: projection.m,v 1.3 2004/10/21 07:16:47 johanl Exp $

if nargin<2
    error('Not enough input arguments.')
end

f = [];
for i = 1:length(F)
    if  F.clauses{i}{1}.type==2
        fi =  F.clauses{i}{1}.data;
        f = [f;fi(:)];
    else
        error('Only linear element-wise inequalities can be projected')
    end
end

if ~islinear(F)
    error('Only linear element-wise inequalities can be projected')
end

B = full(getbase(f));
P = polytope(-B(:,2:end),B(:,1));
y_vars = getvariables(F);

x_vars = getvariables(x);

if ~isempty(setdiff(x_vars,y_vars))
    error('Huh? Project on what! Some of those variables are not part of the original polyhedron');
end

x = recover(x_vars);

x_in_y = find(ismember(x_vars,y_vars));

if nargin == 2
    P = projection(P,x_in_y);
else
    Options.projection=method;
    P = projection(P,x_in_y,Options);
end

H = get(P,'H');
K = get(P,'K');

F = set(H*x < K);
