function output = callclp(interfacedata)

% Author Johan Lfberg 
% $Id: callclp.m,v 1.3 2004/10/28 14:50:58 johanl Exp $

% Retrieve needed data
options = interfacedata.options;
F_struc = interfacedata.F_struc;
c       = interfacedata.c;
K       = interfacedata.K;
ub      = interfacedata.ub;
lb      = interfacedata.lb;
Q       = interfacedata.Q; 

n = length(c);
% Bounded variables converted to constraints
if ~isempty(ub)
    % TODO : MAKE SURE SIZES ARE CORRECT  
    lb(lb==-inf) = -1e30;
    ub(ub==inf) = 1e30;
else
    lb=[];%repmat(-1e30,n,1);    % just for sure
    ub=[];%repmat(1e30,n,1);         
end

if options.showprogress;showprogress('Calling CLP',options.showprogress);end

options.qsopt.verbose = options.verbose;

if K.f>0
    Aeq = -F_struc(1:K.f,2:end);
    beq = F_struc(1:K.f,1);
else
    Aeq = [];
    beq = [];
end

if K.l > 0
    A = -F_struc(1+K.f:end,2:end);
    b = F_struc(1+K.f:end,1);
else
    A = [];
    b = [];
end

if options.savedebug
    save qsoptdebug
end

% Call mex-interfacec
solvertime = clock; 
[x,lambda,STATUS] = clp(2*Q,c,A,b,Aeq,beq,lb,ub,options.qsopt);
solvertime = etime(clock,solvertime);
problem = 0;

if options.saveduals
    D_struc = -lambda;    
else
    D_struc = [];
end

% Check, currently not exhaustive...
switch STATUS
    case 0
        problem = 0;
    case 1
        problem = 1;
    case 2
        problem = 2;
    otherwise
        problem = -1;
end
infostr = yalmiperror(problem,'CLP');	

% Save all data sent to solver?
if options.savesolverinput
	solverinput = [];	
else
	solverinput = [];
end

% Save all data from the solver?
if options.savesolveroutput
	solveroutput = [];	
else
	solveroutput = [];
end

% Standard interface 
output.x           = x;
output.problem     = problem;
output.infostr     = infostr;
output.solverinput = solverinput;
output.solveroutput= solveroutput;
output.solvertime  = solvertime;
output.D_struc = D_struc;