function output = callcplexint(interfacedata)

% Author Johan Lfberg 
% $Id: callcplexint.m,v 1.3 2004/06/18 15:49:15 johanl Exp $

% Retrieve needed data
options = interfacedata.options;
F_struc = interfacedata.F_struc;
c       = interfacedata.c;
Q       = interfacedata.Q;
K       = interfacedata.K;
x0      = interfacedata.x0;
integer_variables = interfacedata.integer_variables;
binary_variables = interfacedata.binary_variables;
UB      = interfacedata.ub;
LB      = interfacedata.lb;

showprogress('Calling CPLEXINT',options.showprogress);

SENSE = 1;     
C = full(c);   
if K.l+K.f == 0
    A = zeros(1,length(c));A(1)=1;
    B = 1e6;
else
    A =-(F_struc(1:K.f+K.l,2:end)); 
    B = full(F_struc(1:K.f+K.l,1));            
end

INDEQ = [];
if K.f>0
    INDEQ(1:K.f) = 1:K.f;
end

if K.q(1)>0
    top = K.f+K.l + 1;
    for i = 1:length(K.q)
        m = K.q(i)-2;
        f = F_struc(top,1)-1;
        R = F_struc(top+1:top+m,2:end)/2;
        QC(i).Q = full(R'*R);
        QC(i).r = full(F_struc(top,1)-1);
        QC(i).L = full(-F_struc(top,2:end));
        top = top+m+2;
    end
else
    QC = [];
end

VARTYPE = repmat('C',size(A,2),1);
VARTYPE(integer_variables)='I'; % Integer variables
VARTYPE(binary_variables) ='B'; % Binary variables

H = full(2*Q);
if nnz(Q)==0
    H = [];
end

PARAM = [];
OPTIONS.verbose = options.verbose;

if options.savedebug
    save cplexintdebug H C A B LB UB QC VARTYPE INDEQ options OPTIONS
end

% Call mex-interface
solvertime = clock; 
[x,FMIN,SOLSTAT,DETAILS] = cplexint(H, C, A, B, INDEQ, QC, LB, UB,VARTYPE,PARAM,OPTIONS);
solvertime = etime(clock,solvertime);
problem = 0;
D_struc = -DETAILS.dual;    

switch SOLSTAT
    case {1,101,102}
        problem = 0;
    case {3}
        problem = 1;
    case {4,119}
        problem = 12;
    otherwise
        problem = -1;
end
infostr = yalmiperror(problem,'CPLEXINT');	

% Save all data sent to solver?
if options.savesolverinput
    solverinput.H = H;
    solverinput.A = A;
    solverinput.C = C;
    solverinput.INDEQ = INDEQ;
    solverinput.QC = QC;
    solverinput.B = B;
    solverinput.CTYPE = CTYPE;
    solverinput.LB = LB;
    solverinput.UB = UB;
else
    solverinput = [];
end

% Save all data from the solver?
if options.savesolveroutput
    solveroutput.x = x;
    solveroutput.FMIN = FMIN;
    solveroutput.SOLSTAT = SOLSTAT;
    solveroutput.DETAILS=DETAILS;
else
    solveroutput = [];
end


% Standard interface 
output.x           = x;
output.problem     = problem;
output.infostr     = infostr;
output.solverinput = solverinput;
output.solveroutput= solveroutput;
output.solvertime  = solvertime;
output.D_struc = D_struc;