function output = callglpk(interfacedata)

% Author Johan Lfberg 
% $Id: callglpk.m,v 1.6 2004/12/20 05:34:14 johanl Exp $

% Retrieve needed data
options = interfacedata.options;
F_struc = interfacedata.F_struc;
c       = interfacedata.c;
K       = interfacedata.K;
integer_variables = interfacedata.integer_variables;
binary_variables = interfacedata.binary_variables;
ub      = interfacedata.ub;
lb      = interfacedata.lb;

n = length(c);
% Bounded variables converted to constraints
if ~isempty(ub)
    LB = lb;
    UB = ub;
else
    LB=repmat(-1e6,n,1);    % just for sure
    UB=repmat(1e6,n,1);         
end

% GLPK does not like lb==ub
equality_in_bound = find((abs(LB-UB)<1e-12) & ~isinf(LB));
m = length(equality_in_bound);
if ~isempty(equality_in_bound)
    F_struc = [-LB(equality_in_bound) sparse(1:m,equality_in_bound,ones(m,1),m,n);F_struc];
    UB(equality_in_bound) = UB(equality_in_bound) + 1;
    K.f = K.f + m;
end

if options.showprogress;showprogress('Calling GLPK',options.showprogress);end

% GLPK notation for sure...
SENSE = 1;     % Minimize
C = full(c);   % Must be full
A =-F_struc(:,2:end);
B = full(F_struc(:,1));         % Must be full
if length(B)==0;
    A = C';
    B = 1e6;
end
CTYPE = [repmat('S',K.f,1); repmat('U',K.l,1)];
VARTYPE = repmat('C',n,1);
VARTYPE(integer_variables)='I'; 

% Call mex-interface
solvertime = clock; 
if options.savedebug
    save glpkmexdebug
end
options.glpk.msglev = options.verbose;
if options.glpk.msglev==1
    options.glpk.msglev = 2;
end

[x,FMIN,STATUS,LAMBDA_EXTRA] = glpkmex(SENSE,C,A,B,CTYPE,LB,UB,VARTYPE,options.glpk,options.glpk.lpsolver,options.glpk.save);
if isstruct(LAMBDA_EXTRA)
    LAMBDA = LAMBDA_EXTRA.lambda;
    EXTRA = LAMBDA_EXTRA;
else
    LAMBDA = LAMBDA_EXTRA;
    EXTRA = 'Not saved in old GLPKMEX version, update...'; 
end
solvertime = etime(clock,solvertime);
problem = 0;

if options.saveduals
    D_struc = -LAMBDA(m+1:end,:);    
else
    D_struc = [];
end

% Hack
if isempty([ub;lb]) & (any(LB==x & C>0) | any(UB==x & C<0))
    STATUS = 214;
end

% Check, currently not exhaustive...
switch STATUS
    case {171,180,151,200}
        problem = 0;
    case {182,173,183,0,213,204}
        problem = 1;
    case 214
        problem = 2;
    case 207
        problem = 3;
    case 210
        problem = 4;
    case 170
        problem = 11;
    otherwise
        problem = -1;
end
infostr = yalmiperror(problem,'GLPKMEX');	

% Save all data sent to solver?
if options.savesolverinput
	solverinput.A = A;
	solverinput.C = C;
	solverinput.B = B;
	solverinput.CTYPE = CTYPE;
	solverinput.LB = LB;
	solverinput.UB = UB;
    solverinput.param = options.glpk;
    solverinput.lpsolver = options.glpk.lpsolver;
else
	solverinput = [];
end

% Save all data from the solver?
if options.savesolveroutput
	solveroutput.x = x;
    solveroutput.FMIN = FMIN;
    solveroutput.STATUS = STATUS;
    solveroutput.LAMBDA=LAMBDA;
    solveroutput.EXTRA = EXTRA; 
else
	solveroutput = [];
end


% Standard interface 
output.x           = x;
output.problem     = problem;
output.infostr     = infostr;
output.solverinput = solverinput;
output.solveroutput= solveroutput;
output.solvertime  = solvertime;
output.D_struc = D_struc;