function output = callpenbmi(interfacedata);

% Author Johan Lfberg
% $Id: callloqo.m,v 1.1 2004/06/17 08:40:25 johanl Exp $

% Retrieve needed data
options = interfacedata.options;
F_struc = interfacedata.F_struc;
c       = interfacedata.c;
K       = interfacedata.K;
x0      = interfacedata.x0;
nonlins = interfacedata.nonlins;
nonlins2 = interfacedata.nonlins2;
allvariables = interfacedata.allvariables;
ub      = interfacedata.ub;
lb      = interfacedata.lb;


% Bounded variables converted to constraints
if ~isempty(ub)
    [F_struc,K] = addbounds(F_struc,K,ub,lb);
end

if ~isempty(nonlins)
    nonlinearindicies = nonlins(:,1);
else
    nonlinearindicies = [];
end
linearindicies = setdiff(1:length(c),nonlinearindicies);
c  = c(linearindicies);

% Prepare for AMPL
for i = 1:length(linearindicies)
    names{i} = ['x[' num2str(i) ']'];
end
for i = 1:size(nonlins2,1)
    % Order
    r = max(find(nonlins2(i,:)))-1;
    name = '';
    for j = 2:r
        name = [name names{nonlins2(i,j)} '*'];
    end
    name = [name names{nonlins2(i,r+1)}];
    names{nonlins2(i,1)} = name;
end

obj = '';
nz = find(c);
for j = 1:length(nz)-1
    obj = [obj num2str(c(nz(j))) '*' names{nz(j)} '+'];        
end
obj = [obj num2str(c(nz(end))) '*' names{nz(end)}];   

for i = 1:1:K.l
    cstr{i} = ['0 <= ' num2str(F_struc(i,1)) '+'];
    nz = find(F_struc(i,2:end));
    for j = 1:length(nz)-1
        cstr{i} = [cstr{i} num2str(F_struc(i,nz(j)+1)) '*' names{nz(j)} '+'];        
    end
    cstr{i} = [cstr{i} num2str(F_struc(i,nz(end)+1)) '*' names{nz(end)}];   
end

% Generate AMPL file
filename = tempname;
fid = fopen(filename,'w');
fprintf(fid,['var x {1..%i};\r\n'],length(c));
fprintf(fid,['minimize obj:  ' obj ';']);
fprintf(fid,'\r\n');
for i = 1:1:K.l
    fprintf(fid,['subject to constr%i: ' cstr{i} ';\r\n'],i);
end
fprintf(fid,'option solver loqo;\r\n'); 
fprintf(fid,'solve;\r\n'); 
fprintf(fid,'display x;\r\n'); 
fprintf(fid,'display obj;\r\n'); 
fclose(fid);

% Run solver
[dummy,output] = dos(['ampl ' filename]);   
lrobj = str2num(output(findstr(output,'obj = ')+6:end));
        
% Try to find output
start = findstr(output,'x [*]');
thend = findstr(output(start:end),';');
Z=str2num(output(start+9:start+thend-2));
x = Z(:,2);
problem=0;
infostr = yalmiperror(problem);

% Standard interface 
output.x           = x;
output.problem     = problem;
output.infostr     = infostr;
output.solverinput = [];
output.solveroutput= [];
output.solvertime  = 0;
output.D_struc     = [];
return


if ~isempty(nonlins)
    lhs = nonlins(:,1);
    rhs = [nonlins(:,2);nonlins(:,3)];
    if ~isempty(find(ismember(lhs,rhs)))
        % Standard interface 
        output.x           = repmat(nan,length(linearindicies),1);
        output.problem     = -2;
        output.infostr     = yalmiperror(-2,'PENBMI');
        output.solverinput = [];
        output.solveroutput= [];
        output.solvertime  = 0;
        output.D_struc     = [];
        return
    end
end

% Any non-linear scalar inequalities?
% Move these to the BMI part
if K.l>0
    nonlinear_scalars = find(any(full(F_struc(1:K.l,[nonlinearindicies(:)'+1])),2));
    if ~isempty(nonlinear_scalars)
        Kold = K;
        linear_scalars = setdiff(1:K.l,nonlinear_scalars);
        F_struc = [F_struc(linear_scalars,:);F_struc(nonlinear_scalars,:);F_struc(K.l+1:end,:)];
        K.l = K.l-length(nonlinear_scalars);
        if (length(K.s)==1) & (K.s==0)
            K.s = [repmat(1,1,length(nonlinear_scalars))];
        else
            K.s = [repmat(1,1,length(nonlinear_scalars)) K.s];
        end
    end
end

pen = sedumi2pen(F_struc(:,[1 linearindicies(:)'+1]),K,c,x0);

if ~isempty(nonlinearindicies)
    bmi = sedumi2pen(F_struc(:,[nonlinearindicies(:)'+1]),K,[],[]);
    pen.ki_dim = bmi.ai_dim;
    % Nonlinear index
    pen.ki_dim = bmi.ai_dim;
    pen.ki_row = bmi.ai_row;
    pen.ki_col = bmi.ai_col;
    pen.ki_nzs = bmi.ai_nzs;
    pen.ki_val = bmi.ai_val;
    for i = 1:length(bmi.ai_idx)
        nl = nonlinearindicies(1+bmi.ai_idx(i));
        indx = find(nonlins(:,1)==nl);
        v1 = nonlins(indx,2);
        v2 = nonlins(indx,3);
        pen.ki_idx(i)=v1;
        pen.kj_idx(i)=v2;
    end
else
    pen.ki_dim = 0*pen.ai_dim;
    pen.ki_row = 0;
    pen.ki_col = 0;
    pen.ki_nzs = 0;
    pen.ki_idx = 0;
    pen.kj_idx = 0;
end

ops = struct2cell(options.penbmi);ops = [ops{1:end}];
pen.ioptions = ops(1:8);
pen.foptions = ops(9:end);
pen.ioptions(4) = options.verbose;

if ~isempty(x0)
    pen.x0 = x0(linearindicies);
    pen.x0 = pen.x0(:)';
end

solvertime = clock; 
[f,xout,u,iflag,niter,feas] = penbmim(pen);
solvertime = etime(clock,solvertime);

% Get dual variable
% First, get the nonlinear scalars treated as BMIs
if exist('nonlinear_scalars') 
    if ~isempty(nonlinear_scalars)
        n_orig_scalars = length(nonlinear_scalars)+K.l;
        linear_scalars = setdiff(1:n_orig_scalars,nonlinear_scalars);
        u_nonlinear=u(K.l+1:K.l+length(nonlinear_scalars));
        u(K.l+1:K.l+length(nonlinear_scalars))=[];
        u_linear = u(1:K.l);
        u_scalar = zeros(1,n_orig_scalars);
        u_scalar(linear_scalars)=u_linear;
        u_scalar(nonlinear_scalars)=u_nonlinear;
        u = [u_scalar u(1+K.l:end)];
        K = Kold;
    end
end

u = u(:);
D_struc = u(1:1:K.l);
if length(K.s)>0
    if K.s(1)>0
        pos = K.l+1;
        for i = 1:length(K.s)
            temp = zeros(K.s(i),K.s(i));
            vecZ = u(pos:pos+0.5*K.s(i)*(K.s(i)+1)-1);
            top = 1;
            for j = 1:K.s(i)
                len = K.s(i)-j+1;
                temp(j:end,j)=vecZ(top:top+len-1);top=top+len;
            end
            temp = (temp+temp');j = find(speye(K.s(i)));temp(j)=temp(j)/2;
            D_struc = [D_struc;temp(:)];
            pos = pos + (K.s(i)+1)*K.s(i)/2;
        end
    end
end


%Recover solution
if isempty(nonlins)
    x = xout(:);
else
    x = zeros(length(allvariables),1);
    for i = 1:length(linearindicies)
        x(linearindicies(i)) = xout(i);
    end
end

problem = 0;
switch iflag
    case 0 
        problem = 0; % OK
    case {1,3}
        problem = 4;
    case 2
        problem = 1; % INFEASIBLE
    case 4
        problem = 3; % Numerics
    case 5
        problem = 7;
    case {6,7}
        problem = 11;
    otherwise
        problem = -1;
end
infostr = yalmiperror(problem,'PENBMI');	

if options.savesolveroutput
    solveroutput.Result = Result;
else
    solveroutput = [];
end

if options.savesolverinput
    solverinput.Prob = Prob;
else
    solverinput = [];
end

% Standard interface 
output.x           = x;
output.problem     = problem;
output.infostr     = infostr;
output.solverinput = solverinput;
output.solveroutput= solveroutput;
output.solvertime  = solvertime;
output.D_struc     = D_struc;