function output = callmiqp(interfacedata)

% Author Johan Lfberg 
% $Id: callmiqp.m,v 1.1 2004/06/17 08:40:26 johanl Exp $

% Retrieve needed data
options = interfacedata.options;
F_struc = interfacedata.F_struc;
c       = interfacedata.c;
Q       = interfacedata.Q;
K       = interfacedata.K;
x0      = interfacedata.x0;
binary_variables = interfacedata.binary_variables;

D_struc = [];
    
showprogress('Calling MIQP',options.showprogress);

if isempty(F_struc)
    Aeq = [];
    beq = [];
    A = [];
    b = [];
else
    Aeq = -F_struc(1:1:K.f,2:end);
    beq = F_struc(1:1:K.f,1);        
    A =-F_struc(K.f+1:end,2:end);
    b = F_struc(K.f+1:end,1);   
end

vartype = repmat('C',size(A,2),1);
vartype(binary_variables)='B'; 

% Call mex-interface
solvertime = clock; 
[x,fmin,flag,extendedflag] = miqp(2*full(Q), full(c), full(A), full(b), full(Aeq), full(beq), vartype, [], [], [],options.miqp);
solvertime = etime(clock,solvertime);
problem = 0;

% Check, currently not exhaustive...
switch flag
    case {-1}
        infostr = yalmiperror(2,'MIQP');
    case 1
        infostr = yalmiperror(0,'MIQP');
    case {5,7}
        problem = 1;
        infostr = yalmiperror(1,'MIQP');
    case {11,15}
        problem = 3;
        infostr = yalmiperror(1,'MIQP');
    otherwise
        problem = -1;
        infostr = yalmiperror(-1,'MIQP');	
end

% Save all data sent to solver?
if options.savesolverinput
	solverinput.A = A;
	solverinput.b = b;
	solverinput.Aeq = Aq;
	solverinput.beq = beq;
	solverinput.c = c;
	solverinput.H = Q;
    solverinput.vartype = vartype;
    solverinput.options = options.miqp;
else
	solverinput = [];
end

% Save all data from the solver?
if options.savesolveroutput
	solveroutput.x = x;
    solveroutput.fmin = fmin;
    solveroutput.flag = flag;
    solveroutput.extendedflag=extendedflag;
else
	solveroutput = [];
end


% Standard interface 
output.x           = x;
output.problem     = problem;
output.infostr     = infostr;
output.solverinput = solverinput;
output.solveroutput= solveroutput;
output.solvertime  = solvertime;
output.D_struc = D_struc;