function output = callmpt(interfacedata)

% Author Johan Lfberg 
% $Id: callmpt.m,v 1.1 2004/06/17 08:40:26 johanl Exp $

% Retrieve needed data
options = interfacedata.options;
F_struc = interfacedata.F_struc;
c       = interfacedata.c;
K       = interfacedata.K;
x0      = interfacedata.x0;
Q       = interfacedata.Q;
lb      = interfacedata.lb;
ub      = interfacedata.ub;

param_var= interfacedata.parametric_variables;
free_var = setdiff(1:length(c),param_var);

F = Q(param_var,free_var);
Y = 0.5*Q(param_var,param_var);
H = Q(free_var,free_var);
A = -F_struc(:,1+free_var);
B = -F_struc(:,1+param_var);
b = F_struc(:,1);
Cf = 0.5*c(free_var)';
Cx = 0.5*c(param_var)';

% Row for exploration constraints
only_param = (sum(abs([full(A)]),2)==0);

G = A(~only_param,:);
W = b(~only_param);
E = -B(~only_param,:);

Matrices.G=full(G);
Matrices.E=full(E);	     
Matrices.W=full(W);
Matrices.H=full(H);
Matrices.F=full(F);
Matrices.Y=full(Y);
Matrices.Cf = full(Cf);
Matrices.Cx = full(Cx);
Matrices.Cc = interfacedata.f;

% Exploration space
Matrices.bndA=full(B(only_param,:));
Matrices.bndb=full(b(only_param));

if nnz(Q)==0
    % Whoops, it's an LP
    Matrices.H = c(free_var)';
end

if options.savedebug
save mptdebug Matrices
end
showprogress('Calling MPT',options.showprogress);
solvertime = clock; 
if nnz(Q)==0
    [Pn,Fi,Gi,activeConstraints,Phard] = mpt_mplp(Matrices);   
else
    [Pn,Fi,Gi,activeConstraints,Phard] = mpt_mpqp(Matrices);
end
solvertime = etime(clock,solvertime);
problem = 0;

% Internal format for duals
D_struc = [];

problem = 0;
infostr = yalmiperror(problem,'MPT');       

% Save all data sent to solver?
if options.savesolverinput
    solverinput.Matrices = Matrices;
    solverinput.options  = [];
else
    solverinput = [];
end

% Save all data from the solver?
if options.savesolveroutput
    solveroutput.Pn = Pn;
    solveroutput.Fi = Fi;
    solveroutput.Gi = Gi;
    solveroutput.activeConstraints = activeConstraints;
    solveroutput.Phard=Phard;  
else
    solveroutput = [];
end

% Standard interface 
output.x           = nan*ones(length(c),1);
output.problem     = problem;
output.infostr     = infostr;
output.solverinput = solverinput;
output.solveroutput= solveroutput;
output.solvertime  = solvertime;
output.D_struc = D_struc;