function output = callpenbmi(interfacedata);

% Author Johan Lfberg
% $Id: callpenbmim.m,v 1.7 2004/11/18 13:11:45 johanl Exp $

% Memory bug in penbmim causes segmentation fault unless we clear penbmim.dll
%clear penbmim
% Retrieve needed data
options = interfacedata.options;
F_struc = interfacedata.F_struc;
c       = interfacedata.c;
Q       = interfacedata.Q;
K       = interfacedata.K;
x0      = interfacedata.x0;
monomtable = interfacedata.monomtable;
ub      = interfacedata.ub;
lb      = interfacedata.lb;

% Linear before bilinearize
temp = sum(monomtable,2)>1;
tempnonlinearindicies = find(temp);
templinearindicies = find(~temp);

% Bilinearize
added_constraints_from_bilinearization = 0;
%[added_constraints_from_bilinearization,F_struc,c,Q,monomtable,K] = bilinearize(F_struc,c,Q,monomtable,K);

% Bounded variables converted to constraints
if ~isempty(ub)
    [F_struc,K] = addbounds(F_struc,K,ub,lb);
end

% This one only occurs if called from bmibnb
if K.f>0
    F_struc = [-F_struc(1:K.f,:);F_struc];
    F_struc(1:K.f,1) = F_struc(1:K.f,1)+sqrt(eps);
    K.l = K.l + 2*K.f;
    K.f = 0;
end

if isempty(monomtable)
    monomtable = eye(length(c));
end

temp = sum(monomtable,2)>1;
nonlinearindicies = find(temp);
linearindicies = find(~temp);
c0 = c;
c  = c(linearindicies);
Q = Q(linearindicies,linearindicies);
nonlinear_scalars = [];

% Any non-linear scalar inequalities?
% Move these to the BMI part
if K.l>0
    nonlinear_scalars = find(any(full(F_struc(1:K.l,[nonlinearindicies(:)'+1])),2));
    if ~isempty(nonlinear_scalars)
        Kold = K;
        linear_scalars = setdiff(1:K.l,nonlinear_scalars);
        F_struc = [F_struc(linear_scalars,:);F_struc(nonlinear_scalars,:);F_struc(K.l+1:end,:)];
        K.l = K.l-length(nonlinear_scalars);
        if (length(K.s)==1) & (K.s==0)
            K.s = [repmat(1,1,length(nonlinear_scalars))];
        else
            K.s = [repmat(1,1,length(nonlinear_scalars)) K.s];
        end
    end
end

if ~isempty(F_struc)
    pen = sedumi2pen(F_struc(:,[1 linearindicies(:)'+1]),K,c,x0);
else
    pen = sedumi2pen([],K,c,x0);
end

if ~isempty(nonlinearindicies)
    bmi = sedumi2pen(F_struc(:,[nonlinearindicies(:)'+1]),K,[],[]);
    pen.ki_dim = bmi.ai_dim;
    % Nonlinear index
    pen.ki_dim = bmi.ai_dim;
    pen.ki_row = bmi.ai_row;
    pen.ki_col = bmi.ai_col;
    pen.ki_nzs = bmi.ai_nzs;
    pen.ki_val = bmi.ai_val;
    for i = 1:length(bmi.ai_idx)
        nl = nonlinearindicies(1+bmi.ai_idx(i));
        v = find(monomtable(nl,:));
        if length(v)==1
            v(2)=v(1);
        end      
        pen.ki_idx(i)=find(linearindicies == v(1));
        pen.kj_idx(i)=find(linearindicies == v(2));
    end
else
    pen.ki_dim = 0*pen.ai_dim;
    pen.ki_row = 0;
    pen.ki_col = 0;
    pen.ki_nzs = 0;
    pen.ki_idx = 0;
    pen.kj_idx = 0;
    pen.kj_val = 0;    
end

if nnz(Q)>0
    [row,col,vals] = find(triu(Q));
    pen.q_nzs = length(row);
    pen.q_val = vals';
    pen.q_col = col'-1;
    pen.q_row = row'-1;
else
    pen.q_nzs = 0;
    pen.q_val = 0;
    pen.q_col = 0;
    pen.q_row = 0;    
end

ops = struct2cell(options.penbmi);ops = [ops{1:end}];
pen.ioptions = ops(1:12);
pen.foptions = ops(13:end);
pen.ioptions(4) = max(0,min(3,options.verbose+1));
if pen.ioptions(4)==1
    pen.ioptions(4)=0;
end

% ****************************************
% UNCOMMENT THIS FOR PENBMI version 1
% ****************************************
pen.ioptions = pen.ioptions(1:8);
pen.foptions = pen.foptions(1:8);

if ~isempty(x0)    
    pen.x0(isnan(pen.x0)) = 0;
    pen.x0 = x0(linearindicies);    
    pen.x0 = pen.x0(:)';
end

if options.savedebug
    save penbmimdebug pen
end

if options.showprogress;showprogress('Calling PENBMI',options.showprogress);end

solvertime = clock; 
[f,xout,u,iflag,niter,feas] = penbmim(pen);
solvertime = etime(clock,solvertime);

if options.saveduals
    
    % Get dual variable
    % First, get the nonlinear scalars treated as BMIs
    if ~isempty(nonlinear_scalars)
        n_orig_scalars = length(nonlinear_scalars)+K.l;
        linear_scalars = setdiff(1:n_orig_scalars,nonlinear_scalars);
        u_nonlinear=u(K.l+1:K.l+length(nonlinear_scalars));
        u(K.l+1:K.l+length(nonlinear_scalars))=[];
        u_linear = u(1:K.l);
        u_scalar = zeros(1,n_orig_scalars);
        u_scalar(linear_scalars)=u_linear;
        u_scalar(nonlinear_scalars)=u_nonlinear;
        u = [u_scalar u(1+K.l:end)];
        K = Kold;
    end
       
    K.l = K.l - added_constraints_from_bilinearization;
    u(1:1:added_constraints_from_bilinearization) = [];
    u = u(:);
    D_struc = u(1:1:K.l);
    if length(K.s)>0
        if K.s(1)>0
            pos = K.l+1;
            for i = 1:length(K.s)
                temp = zeros(K.s(i),K.s(i));
                vecZ = u(pos:pos+0.5*K.s(i)*(K.s(i)+1)-1);
                top = 1;
                for j = 1:K.s(i)
                    len = K.s(i)-j+1;
                    temp(j:end,j)=vecZ(top:top+len-1);top=top+len;
                end
                temp = (temp+temp');j = find(speye(K.s(i)));temp(j)=temp(j)/2;
                D_struc = [D_struc;temp(:)];
                pos = pos + (K.s(i)+1)*K.s(i)/2;
            end
        end
    end
else
    D_struc = [];
end

%Recover solution
if isempty(nonlinearindicies)
    x = xout(:);
else
    x = zeros(length(interfacedata.c),1);
    %xout = xout(1:size(interfacedata.monomtable,1));
    for i = 1:length(templinearindicies)
        x(templinearindicies(i)) = xout(i);
    end
end

problem = 0;
switch iflag
    case 0 
        problem = 0; % OK
    case {1,3}
        problem = 4;
    case 2
        problem = 1; % INFEASIBLE
    case 4
        problem = 3; % Numerics
    case 5
        problem = 7;
    case {6,7}
        problem = 11;
    otherwise
        problem = -1;
end
infostr = yalmiperror(problem,'PENBMI/PENOPT');	

if options.savesolveroutput
    solveroutput.pen = pen;
else
    solveroutput = [];
end

if options.savesolverinput
    solverinput.pen = pen;
else
    solverinput = [];
end

% Standard interface 
output.x           = x;
output.problem     = problem;
output.infostr     = infostr;
output.solverinput = solverinput;
output.solveroutput= solveroutput;
output.solvertime  = solvertime;
output.D_struc     = D_struc;




























% HORRIBLE HACK TO BILINEARIZE PROBLEM
% FIX : Well, make it correcto begin with, then fast....
function [added_constraints_from_bilinearization,F_struc,c,Q,monomtable,K] = bilinearize(F_struc,c,Q,monomtable,K)

added_constraints_from_bilinearization = 0;
if any(sum(monomtable,2)>2)
    
    
    still_large_terms = find(sum(monomtable>=3,2)) ;
    while ~isempty(still_large_terms)        
        large_term = still_large_terms(1);
        this_is_large = find(monomtable(large_term,:));
        this_is_large = this_is_large(1);
        power = monomtable(large_term,this_is_large);
        p1 = floor(power/2);
        p2 = ceil(power/2);
        v = zeros(1,size(monomtable,2));
        v(this_is_large) = p1;
        var_p1 = findrows(monomtable,v);
        if isempty(var_p1)
            var_p1 = size(monomtable,1)+1;
            monomtable(end+1,this_is_large) = p1;
        end
        v = zeros(1,size(monomtable,2));
        v(this_is_large) = p2;
        var_p2 = findrows(monomtable,v);
        if isempty(var_p2)
            var_p2 = size(monomtable,1)+1;
            monomtable(end+1,this_is_large) = p2;
        end
        if size(monomtable,1)>size(monomtable,2)
            monomtable(end,size(monomtable,1)) = 0;
        end
        if p1>1
            var_z =size(monomtable,2)+1;
            monomtable(end+1,end+1) = 1;
        else
            var_z = var_p1;
        end
        if p1~=p2
            if p2>1
                var_y =size(monomtable,2)+1;
                monomtable(end+1,end+1) = 1;
            else
                var_y = var_p2;
            end
        else
            var_y = var_z;
        end
        monomtable(large_term,this_is_large) = 0;
        monomtable(large_term,var_z) = monomtable(large_term,var_z) + 1;
        monomtable(large_term,var_y) = monomtable(large_term,var_y) + 1;
        if var_p1~=var_z
        v = spalloc(2,1+size(monomtable,1),2);
        v(1,1+var_z) = 1;
        v(1,1+var_p1) = -1;
        v(2,1+var_z) = -1;
        v(2,1+var_p1) = 1;
        F_struc(1,1+size(monomtable,1))=0;
        F_struc = [v;F_struc];
        K.l = K.l + 2;
        added_constraints_from_bilinearization = added_constraints_from_bilinearization  + 2;
        end
        
        if p1~=p2
            if var_p2~=var_y
                v = spalloc(2,1+size(monomtable,1),2);
                v(1,1+var_y) = 1;
                v(1,1+var_p2) = -1;
                v(2,1+var_y) = -1;
                v(2,1+var_p2) = 1;
                F_struc(1,1+size(monomtable,1))=0;
                F_struc = [v;F_struc];
                K.l = K.l + 2;
                added_constraints_from_bilinearization = added_constraints_from_bilinearization  + 2;
            end
        end
        if var_p1~=var_z
        c(var_z)=0;
        Q(var_z,var_z)=0;
        end
        if var_p2~=var_y
        c(var_y)=0;
        Q(var_z,var_z)=0;
        end
        still_large_terms = find(sum(monomtable>=3,2)) ;
    end

    
    still_multiple_terms = find(sum(monomtable,2)>2) ;
    while ~isempty(still_multiple_terms)  
        still_multiple_terms = still_multiple_terms(1);
        vars =find(monomtable(still_multiple_terms,:));
        mid = floor(length(vars)/2);
        var1 = vars(1:mid);        
        var2 = vars(mid+1:end);
        
        
        
        %%%%%%%%%%%%%%%%%%%555
        v = zeros(1,size(monomtable,2));
        v(var1) = monomtable(still_multiple_terms,var1);
        var_p1 = findrows(monomtable,v);
        if isempty(var_p1)
            var_p1 = size(monomtable,1)+1;
            monomtable(end+1,var1) = monomtable(still_multiple_terms,var1);
        end
        v = zeros(1,size(monomtable,2));
        v(var2) = monomtable(still_multiple_terms,var2);
        var_p2 = findrows(monomtable,v);
        if isempty(var_p2)
            var_p2 = size(monomtable,1)+1;
            monomtable(end+1,var2) = monomtable(still_multiple_terms,var2);
        end
        if size(monomtable,1)>size(monomtable,2)
            monomtable(end,size(monomtable,1)) = 0;
        end
        if sum(monomtable(var_p1,:))>1
            var_z =size(monomtable,2)+1;
            monomtable(end+1,end+1) = 1;
        else
            var_z = var_p1;
        end
        
            if sum(monomtable(var_p2,:))>1
                var_y =size(monomtable,2)+1;
                monomtable(end+1,end+1) = 1;
            else
                var_y = var_p2;
            end
        monomtable(still_multiple_terms,var1) = 0;
        monomtable(still_multiple_terms,var2) = 0;
        monomtable(still_multiple_terms,var_z) = monomtable(still_multiple_terms,var_z) + 1;
        monomtable(still_multiple_terms,var_y) = monomtable(still_multiple_terms,var_y) + 1;
        if var_p1~=var_z
        v = spalloc(2,1+size(monomtable,1),2);
        v(1,1+var_z) = 1;
        v(1,1+var_p1) = -1;
        v(2,1+var_z) = -1;
        v(2,1+var_p1) = 1;
        F_struc(1,1+size(monomtable,1))=0;
        F_struc = [v;F_struc];
        K.l = K.l + 2;
        added_constraints_from_bilinearization = added_constraints_from_bilinearization  + 2;
        end
        
       
            if var_p2~=var_y
                v = spalloc(2,1+size(monomtable,1),2);
                v(1,1+var_y) = 1;
                v(1,1+var_p2) = -1;
                v(2,1+var_y) = -1;
                v(2,1+var_p2) = 1;
                F_struc(1,1+size(monomtable,1))=0;
                F_struc = [v;F_struc];
                K.l = K.l + 2;
                added_constraints_from_bilinearization = added_constraints_from_bilinearization  + 2;
            end
       
        if var_p1~=var_z
        c(var_z)=0;
        Q(var_z,var_z)=0;
        end
        if var_p2~=var_y
        c(var_y)=0;
        Q(var_y,var_y)=0;
        end
        %%%%%%%%%%%%%%%%%%%%%%
        
        
        still_multiple_terms = find(sum(monomtable,2)>2) ;
    end
   
end
