function output = callsedumi(interfacedata)

% Author Johan Lfberg 
% $Id: callsedumi.m,v 1.5 2004/09/16 23:32:41 johanl Exp $

% Retrieve needed data
options = interfacedata.options;
F_struc = interfacedata.F_struc;
c       = interfacedata.c;
K       = interfacedata.K;
ub      = interfacedata.ub;
lb      = interfacedata.lb;

% Create the parameter structure
pars = options.sedumi;
pars.fid = double(options.verbose);

% *********************************************
% Bounded variables converted to constraints
% N.B. Only happens when caller is BNB
% *********************************************
if ~isempty(ub)
    [F_struc,K] = addbounds(F_struc,K,ub,lb);
end

if 0%options.reduce 
    
    b = -c;
    
    % Precalc
    indicies = [0 cumsum(K.s.^2)];
    m = length(K.s);
    
    % ***************************************        
    % Get problem data
    % ***************************************        
    for i = 1:m
        n = K.s(i);
        Bi = getbase(sdpvar(K.s(i),K.s(i)));Bi = Bi(:,2:end);B{i} = Bi;
        nz(i) = n*(n+1)/2;
        A{i} = -F_struc(indicies(i)+1:indicies(i+1),2:end);
        E{i} = A{i}'*B{i};
        C{i} = reshape(F_struc(indicies(i)+1:indicies(i+1),1),n,n);
    end
    
    % ***************************************    
    % Find separable variables
    % ***************************************  
    %     tic
    %     var_in_constraint = zeros(length(c),m);
    %     for i = 1:length(c)
    %         for j = 1:m
    %             var_in_constraint(i,j) = any(E{j}(i,:));
    %         end
    %     end
    %     toc
    %    
    var_in_constraint = zeros(length(c),m);
    for j = 1:m
        var_in_constraint(:,j)   = full(any(E{j}(:,:),2));%any(E{j}(:,:),1)';
    end
    
    
    % ***************************************
    % Begin with a basis for the separable part
    % of the primal variables
    % ***************************************
    all_z = [];
    for i = 1:m
        only_in_this_constraint = find((var_in_constraint(:,i)==1) & sum(var_in_constraint,2)==1);
        if ~isempty(only_in_this_constraint)
            [z_0{i},H{i}]=solveequalities(full(E{i}(only_in_this_constraint,:)),b(only_in_this_constraint));
            x{i} = sdpvar(size(H{i},2),1);
            z{i} = z_0{i}+ H{i}*x{i}; 
            all_z = [all_z;z{i}];
        else
            H{i} = eye(nz(i),nz(i));
            x{i} = sdpvar(nz(i),1);
            z{i} =  x{i};
            all_z = [all_z;z{i}];
        end
    end
    
    % ***************************************
    % Basis with complicating variables
    % ***************************************
    in_many = find(sum(var_in_constraint,2)>1);
    allE = [E{:}];
    b_new = b(in_many) - allE(in_many,:)*getbasematrix(all_z,0);
    E_new = allE(in_many,:)*blkdiag(H{:});
    [x_new,H_new] = solveequalities(full(E_new),full(b_new));
    %    H_new = null(full(E_new));
    %    x_new = E_new\b_new;
    H_total = blkdiag(H{:})*H_new;
    z0_total =getbasematrix(all_z,0) +  blkdiag(H{:})*x_new;
    
    % ***************************************    
    % Set up optimization problem
    % ***************************************    
    F = set([]);
    top = 1;
    z = sdpvar(size(H_total,2),1);
    obj = 0;
    for i = 1:m
        z_0{i} = z0_total(top:top+nz(i)-1);
        H{i} = H_total(top:top+nz(i)-1,:);
        Z{i} = reshape(B{i}*(z_0{i}+H{i}*z),K.s(i),K.s(i));
        F = F + set(Z{i} > 0);
        obj = obj + C{i}(:)'*Z{i}(:);
        top = top + nz(i);
    end
    
    
    % ***************************************    
    % Solve dual problem
    % ***************************************      
    ops = options;
    ops.reduce = 0;
    ops.savesolveroutput = 1;
    sol = solvesdp(F,obj,ops);
    info = sol.Solveroutput.info;
    solvertime = sol.solvertime;
    
    % ***************************************        
    % Recover primal and dual solution
    % ***************************************        
    D_struc = [];
    x = [];
    for i = 1:m
        D_struc = [D_struc;double(Z{i}(:))];
        X{i} = double(dual(F(i)));
        x = [x;X{i}(:)];
    end
    
    % Recover complicating variables
    c_variables = find(c);
    A2 = F_struc(:,1+c_variables);
    
    % Recover original variables
    top = 1;
    for i = 1:length(K.s)
        n = K.s(i);
        variables{i} = find(sum(F_struc(top:top+K.s(i)^2-1,2:end),1));
        top = top + K.s(i)^2;
    end    
    
    %    full(Z_base(1:9,:)'*F_struc(1:9,2:3))    +full(Z_base(10:18,:)'*F_struc(10:18,2:3))    +full(Z_base(19:27,:)'*F_struc(19:27,2:3))
    
    if 1
        c_variables = [];
        y_s_c_vars = [];
        if 0
            Z_linear = [Z{:}];
            Z_base = getbase(Z_linear(:)); 
            c_variables = find(c);
            top = 1;
            b = [];
            G = [];
            for i = 1:length(K.s)
                Z_base = getbase(Z{i});
                for j = 1:length(getvariables(Z{i}))
                    G = [G;(F_struc(top:top+K.s(i)^2-1,1+c_variables)'*Z_base(:,j+1))'];%%F_struc(top:top+K.s(i)^2-1,1+c_variables)'*Z_base(:,j+1)           
                    b = [b;(X{i}(:)-C{i}(:))'*Z_base(:,j+1)];
                end
                top = top + K.s(i)^2;
            end
            
            for i = 1:size(Z_base,2)
                A_c_vars(top,:) = full(Z_base(:,i)'*F_struc(:,1+c_variables));
                b_c_vars(top,1) = full(Z_base(:,i)'*(F_struc(:,1)-double(x)));
                top = top + 1;
            end
            y_s_c_vars = full(A_c_vars)\full(b_c_vars);
        end
        remaining_variables = setdiff((1:(size(F_struc,2)-1)),c_variables);
        if isempty(c_variables)
            F_struc_remaining = F_struc;
        else
            F_struc_remaining = [F_struc(:,1)+F_struc(:,1+c_variables)*y_s_c_vars F_struc(:,1+ remaining_variables)];
        end
        
        
        % Assume that we can solve Recover original variables
        top = 1;
        for i = 1:length(K.s)
            n = K.s(i);        
            variables{i} = find(sum(F_struc_remaining(top:top+K.s(i)^2-1,2:end),1));
            constraints{i} = i;
            top = top + K.s(i)^2;
        end    
        % Now merge if they use same variables
        i = 1;
        j = 2;
        while i<=length(variables)-1
            common = intersect(variables{i},variables{j});
            if isempty(common)
                j = j + 1;
                if j > length(variables)
                    i = i + 1;
                    j = i + 1;
                end
            else
                variables{i} = union(variables{i},variables{j});
                constraints{i} = union(constraints{i},constraints{j});
                variables = {variables{[1:1:j-1 j+1:1:length(variables)]}};
                constraints = {constraints{[1:1:j-1 j+1:1:length(constraints)]}};
            end
        end
        
        
        for i = 1:length(variables)
            A = [];
            b = [];
            for j = 1:length(constraints{i})
                top = sum(K.s(1:constraints{i}(j)-1).^2)+1;
                n = K.s(j);
                A = [A;-F_struc_remaining(top:top+n^2-1,1+variables{i})];
                b = [b;-F_struc_remaining(top:top+n^2-1,1)+x(top:top+n^2-1,:)];
            end
            y_s_i{i} = (A)\(b);
        end
        y_s = zeros(size(F_struc,2)-1,1);
        y_s(c_variables,1) = y_s_c_vars;
        for i = 1:length(variables)
            y_s(remaining_variables(variables{i}),1) = y_s_i{i}(:);
        end
    else
        y_s = -F_struc(:,2:end)\(x-F_struc(:,1));
    end
else
    
    if options.savedebug
        save sedumidebug F_struc c K pars
    end

    % *********************************************
    % Call SeDuMi
    % *********************************************
    solvertime = clock; 
    showprogress('Calling SeDuMi',options.showprogress);
    problem = 0;   
    [x_s,y_s,info] = sedumi(-F_struc(:,2:end),-c,F_struc(:,1),K,pars);
    solvertime = etime(clock,solvertime);
    
    % *********************************************
    % Dual variable
    % *********************************************
    D_struc = x_s;
end

% We solve dual problem with sedumi...this should fix this
x = y_s; % Primals is what we want
temp = info.pinf;
pinf = info.dinf;
dinf = temp;

% Check for reported errors
if (pinf==0) &  (dinf==0)
    problem = 0; % No problems
end

% We can only report one error, use priorities
if (problem==0) & (pinf==1)
    problem = 1; % Primal infeasability
end

if (problem==0) & (dinf==1)
    problem = 2; % Dual infeasability
end

if (problem==0) & (info.numerr==1) | (info.numerr==2)
    problem = 4; %Numerical problems
end

if (problem==0) & (info.iter >= options.sedumi.maxiter)
    % Did we need exactly maxiter iterations to find optimum
    if (pinf==0) & (dinf==0) & (info.numerr==0)
        problem = 0; % Yes
    else
        problem = 3; % No, we are not optimal yet
    end
end

if (problem==0) & (info.feasratio<0.98) 
    problem = 4; 
end

if (abs(info.feasratio+1)<0.01) & (pinf==0) &  (dinf==0)
    problem = 1; 
end


infostr = yalmiperror(problem,'SeDuMi');

% Save ALL data sent to solver
if options.savesolverinput
    solverinput.A = F_struc(:,2:end);
    solverinput.c = F_struc(:,1);
    solverinput.b = c;
    solverinput.K = K;
    solverinput.pars = pars;
else
    solverinput = [];
end

% Save ALL data from the solution?
if options.savesolveroutput
    solveroutput.x = x_s;
    solveroutput.y = y_s;
    solveroutput.info = info;
else
    solveroutput = [];
end

% Standard interface 
output.x           = x;
output.problem     = problem;
output.infostr     = infostr;
output.solverinput = solverinput;
output.solveroutput= solveroutput;
output.solvertime  = solvertime;
output.D_struc = D_struc;


function [x_equ,H,A_equ,b_equ] = solveequalities(A_equ,b_equ)

% Find feasible sing method from Nocedal-Wright book

used_rows = find(sum(abs(A_equ),2));
A1 = A_equ(used_rows,:);
b1 = b_equ(used_rows);
[Q,R] = qr(A1');
n = max(find(sum(abs(R),2)));
Q1 = Q(:,1:n);
R = R(1:n,:);
x_equ = Q1*(R'\b1);  
H = Q(:,n+1:end); % New basis
%x_equ = zeros(size(A_equ,1),1);
%x_equ(used_rows) = x1;
return




[Q,R] = qr(A_equ');
n = max(find(sum(abs(R),2)));
Q1 = Q(:,1:n);
R = R(1:n,:);
x_equ = Q1*(R'\b_equ);  
H = Q(:,n+1:end); % New basis

function [F_struc,newb,K] = primalreduce(F_struc,c,K);


b = -c;
top = 1;
newb = [];
newA = [];
newC = [];
for i = 1:length(K.s)
    n = K.s(i);
    A{i} = -(F_struc(top:top+n^2-1,2:end));
    C{i} = (F_struc(top:top+n^2-1,1));
    Fi = getbase(sdpvar(K.s(i),K.s(i)));F{i} = Fi(:,2:end);
    tic
    [w_0{i},H{i}]=solveequalities(full(A{i}'*F{i}),b);
    toc
    % AA = A{i}'*F{i};
    % H{i} = null(full(AA));
    % w_0{i} = AA\b;
    % norm(AA*H{i},'inf')
    %H{i} = null(full(AA));
    FH{i} = F{i}*H{i};
    newb = [newb;-(C{i}(:)'*FH{i})'];
    newC = [newC;F{i}*w_0{i};];
    newA = blkdiag(newA,-FH{i});
    top = top + n^2;
end

F_struc = [newC newA];