function solver = solverdefinitions

% ****************************************
% DESCRIPTION OF GENERAL SOLVER INTERFACE
% ****************************************
%solver(i).tag     =   name in YALMIP
%solver(i).version =   additional identifier
%solver(i).bmi     =   Bilinear constraints
%solver(i).pmi     =   Polynomial constraint
%solver(i).maxdet  =   MAXDET objective
%solver(i).sdp     =   SDP constraints
%solver(i).kyp     =   KYP constraints
%solver(i).lp      =   Element-wise constraints
%solver(i).qp      =   Quadratic objective functions
%solver(i).gp      =   Sigmonial objective/constraints
%solver(i).ncqp    =   NOT USED
%solver(i).socp    =   Second order constraints
%solver(i).rotsocp =   Rotated second order constraints
%solver(i).equalities= Equality constraints
%solver(i).duals   =   Returns dual variables
%solver(i).complex =   Handles complex data
%solver(i).integer =   Handles integer variables
%solver(i).binary  =   Handles binary variables
%solver(i).parametric  =   Handles parametric variables
%solver(i).checkfor=   File to look for to see if solver is available
%solver(i).call    =   Interface code
%solver(i).subcall =   NOT USED


% ****************************
% INITIALIZE COUNTER
% ****************************
i = 1;

% ****************************
% DEFINE SOLVERS
% ****************************
solver(i).tag     = 'glpk';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 1;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'glpkmex.m'};
solver(i).call    = 'callglpk';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'cdd';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).qc = 0;
solver(i).rotsocp = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'cddmex'};
solver(i).call    = 'callcdd';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'nag';
solver(i).version = 'e04mbf';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'e04mbf'};
solver(i).call    = 'callnage04mbf';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'nag';
solver(i).version = 'e04naf';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'e04naf'};
solver(i).call    = 'callnage04naf';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;


solver(i).tag     = 'xpress';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 1;
solver(i).binary  = 1;
solver(i).parametric = 0;
solver(i).checkfor= {'mexpress.m'};
solver(i).call    = 'callmexpress';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'qsopt';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'qsopt.m'};
solver(i).call    = 'callqsopt';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'clp';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'clp.m'};
solver(i).call    = 'callclp';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'mosek';
solver(i).version = 'LP/QP';
solver(i).bmi     = 1;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 1;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'mosekopt'};
solver(i).call    = 'callmosek';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'mosek';
solver(i).version = 'SOCP';
solver(i).bmi     = 1;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 1;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'mosekopt'};
solver(i).call    = 'callmosek';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'mosek';
solver(i).version = 'GEOMETRIC';
solver(i).bmi     = 1;
solver(i).pmi     = 0;solver(i).pi     = 1;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 1;
solver(i).po      = 1;
solver(i).ncqp    = 1;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'mosekopt'};
solver(i).call    = 'callmosek';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'cplex';
solver(i).version = 'CPLEXINT';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 1;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 1;
solver(i).binary  = 1;
solver(i).parametric = 0;
solver(i).checkfor= {'cplexint'};
solver(i).call    = 'callcplexint';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'cplex';
solver(i).version = 'CPLEXMEX';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 1;
solver(i).binary  = 1;
solver(i).parametric = 0;
solver(i).checkfor= {'cplexmex'};
solver(i).call    = 'callcplexmex';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'cplex';
solver(i).version = 'LP-CPLEXINT';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'lp_cplex.m'};
solver(i).call    = 'calllp_cplex';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'cplex';
solver(i).version = 'QP-CPLEXINT';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'qp_cplex.m'};
solver(i).call    = 'callqp_cplex';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'cplex';
solver(i).version = 'MILP-CPLEXINT';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 1;
solver(i).binary  = 1;
solver(i).parametric = 0;
solver(i).checkfor= {'milp_cplex.m'};
solver(i).call    = 'callmilp_cplex';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'cplex';
solver(i).version = 'MIQP-CPLEXINT';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 1;
solver(i).binary  = 1;
solver(i).parametric = 0;
solver(i).checkfor= {'miqp_cplex.m'};
solver(i).call    = 'callmiqp_cplex';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'linprog';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'linprog'};
solver(i).call    = 'calllinprog';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'bintprog';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 0;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 1;
solver(i).parametric = 0;
solver(i).checkfor= {'bintprog.m'};
solver(i).call    = 'callbintprog';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'quadprog';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'quadprog'};
solver(i).call    = 'callquadprog';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

% solver(i).tag     = 'ipqp';
% solver(i).version = '';
% solver(i).bmi     = 0;
% solver(i).pmi     = 0;solver(i).pi     = 0;
% solver(i).maxdet  = 0;
% solver(i).sdp     = 0;
% solver(i).kyp     = solver(i).sdp;
% solver(i).lp      = 1;
% solver(i).qp      = 1;
% solver(i).gp      = 0;
% solver(i).ncqp    = 0;
% solver(i).socp    = 0;
% solver(i).rotsocp = 0;
% solver(i).qc = 0;
% solver(i).equalities = 1;
% solver(i).duals   = 0;
% solver(i).complex = 0;
% solver(i).integer = 0;
% solver(i).binary  = 0;
% solver(i).parametric = 0;
% solver(i).checkfor= {'ipqp'};
% solver(i).call    = 'callipqp';
% solver(i).subcall = '';
% solver(i).show    = 1;
% i = i+1;


solver(i).tag     = 'ooqp';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'ooqp.m'};
solver(i).call    = 'callooqp';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'sedumi';
solver(i).version = '1.05';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 1;
solver(i).rotsocp = 1;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 1;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'sedumi.m','ada_pcg.m'};
solver(i).call    = 'callsedumi';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'sedumi';
solver(i).version = '1.03';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 1;
solver(i).rotsocp = 1;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 1;
solver(i).complex = 1;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'sedumi.m','doinfac.m'};
solver(i).call    = 'callsedumi';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'sdpt3';
solver(i).version = '3.1';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 1;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'sqlp','skron','symqmr'};
solver(i).call    = 'callsdpt331';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'sdpt3';
solver(i).version = '3.02';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 1;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'sqlp','skron','schursysolve'};
solver(i).call    = 'callsdpt3302';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'sdpt3';
solver(i).version = '3.0';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 1;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'sqlp','mexexec'};
solver(i).call    = 'callsdpt330';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'sdpa';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'sdpam.m'};
solver(i).call    = 'callsdpa';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'dsdp';
solver(i).version = '5';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'dsdp','dvec'};
solver(i).call    = 'calldsdp5';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'dsdp';
solver(i).version = '4';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'dsdp'};
solver(i).call    = 'calldsdp';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'sdplr';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'sdplr'};
solver(i).call    = 'callsdplr';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'csdp';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'csdp','readsol','writesdpa'};
solver(i).call    = 'callcsdp';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;


solver(i).tag     = 'maxdet';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 1;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 0;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'maxdet.m'};
solver(i).call    = 'callmaxdet';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

% solver(i).tag     = 'specsdp';
% solver(i).version = '';
% solver(i).bmi     = 0;
% solver(i).pmi     = 0;solver(i).pi     = 0;
% solver(i).maxdet  = 0;
% solver(i).sdp     = 1;
% solver(i).kyp     = solver(i).sdp;
% solver(i).lp      = 1;
% solver(i).qp      = 0;
% solver(i).gp      = 0;
% solver(i).ncqp    = 0;
% solver(i).socp    = 0;
% solver(i).rotsocp = 0;
% solver(i).equalities = 0;
% solver(i).duals   = 1;
% solver(i).complex = 0;
% solver(i).integer = 0;
% solver(i).binary  = 0;
%solver(i).parametric = 0;
% solver(i).checkfor= {'SPSDPLSCX'};
% solver(i).call    = 'callspecsdp';
% solver(i).subcall = '';
% solver(i).show    = 1;
% i = i+1;

solver(i).tag     = 'pensdp';
solver(i).version = 'PENOPT';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'pensdpm'};
solver(i).call    = 'callpensdpm';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'pensdp';
solver(i).version = 'TOMLAB';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'pensdp'};
solver(i).call    = 'callpensdp';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'penbmi';
solver(i).version = 'PENOPT';
solver(i).bmi     = 1;
solver(i).pmi     = 1;solver(i).pi     = 1;solver(i).po     = 1;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 1;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'penbmim'};
solver(i).call    = 'callpenbmim';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'penbmi';
solver(i).version = 'TOMLAB';
solver(i).bmi     = 1;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'penbmi'};
solver(i).call    = 'callpenbmi';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'lmilab';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 0;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'setlmis'};
solver(i).call    = 'calllmilab';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'mpt';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 1;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 0;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 1;
solver(i).checkfor= {'mpt_mpqp'};
solver(i).call    = 'callmpt';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'kypd';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 1;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'kypd_solver'};
solver(i).call    = 'callkypd';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'fminsearch';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = 0;
solver(i).lp      = 0;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).po      = 1;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 0;
solver(i).complex = 1;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'fminsearch.m'};
solver(i).call    = 'callfminsearch';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

solver(i).tag     = 'fmincon';
solver(i).version = '';
solver(i).bmi     = 1;
solver(i).pmi     = 1;solver(i).pi     = 1;
solver(i).maxdet  = 0;
solver(i).sdp     = 0;
solver(i).kyp     = 0;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 1;
solver(i).ncqp    = 1;
solver(i).socp    = 1;
solver(i).po      = 1;
solver(i).rotsocp = 1;
solver(i).qc = 1;
solver(i).equalities = 1;
solver(i).duals   = 1;
solver(i).complex = 1;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'fmincon.m'};
solver(i).call    = 'callfmincon';
solver(i).subcall = '';
solver(i).show    = 1;
i = i+1;

% % ***************************************
% % SOMEWHAT MORE COMPLEX DEFINITIONS OF
% % THE INTERNAL MICP SOLVER
% % ***************************************
solver(i).tag     = 'bnb';
solver(i).version = '';
solver(i).bmi     = 0;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = 0;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 1;
solver(i).ncqp    = 1;
solver(i).socp    = 1;
solver(i).rotsocp = 1;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 0;
solver(i).complex = 1;
solver(i).integer = 1;
solver(i).binary  = 1;
solver(i).parametric = 0;
solver(i).checkfor= {'bnb'};
solver(i).call    = 'bnb';
solver(i).subcall = '';
solver(i).show    = 0;
i = i+1;

% ***************************************
% SOMEWHAT MORE COMPLEX DEFINITIONS OF
% THE INTERNAL GLOBAL BMI SOLVER
% ***************************************
solver(i).tag     = 'bmibnb';
solver(i).version = '';
solver(i).bmi     = 1;
solver(i).pmi     = 0;solver(i).pi     = 1;solver(i).po     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = 0;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 1;
solver(i).socp    = 1;
solver(i).rotsocp = 1;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 0;
solver(i).complex = 1;
solver(i).integer = 0;
solver(i).binary  = 1;
solver(i).parametric = 0;
solver(i).checkfor= {'bmibnb'};
solver(i).call    = 'bmibnb';
solver(i).subcall = '';
solver(i).show    = 0;
i = i+1;

solver(i).tag     = 'bmilin';
solver(i).version = '';
solver(i).bmi     = 1;
solver(i).pmi     = 1;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = 0;
solver(i).lp      = 1;
solver(i).qp      = 1;
solver(i).gp      = 0;
solver(i).ncqp    = 1;
solver(i).socp    = 1;
solver(i).rotsocp = 1;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 0;
solver(i).complex = 1;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'callbmilin'};
solver(i).call    = 'callbmilin';
solver(i).subcall = '';
solver(i).show    = 0;
i = i+1;

solver(i).tag     = 'bmialt';
solver(i).version = '';
solver(i).bmi     = 1;
solver(i).pmi     = 0;solver(i).pi     = 0;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 0;
solver(i).duals   = 0;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'callbmialt'};
solver(i).call    = 'callbmialt';
solver(i).subcall = '';
solver(i).show    = 0;
i = i+1;

solver(i).tag     = 'none';
solver(i).version = '';
solver(i).bmi     = 1;
solver(i).pmi     = 1;solver(i).pi     = 1;
solver(i).maxdet  = 0;
solver(i).sdp     = 1;
solver(i).kyp     = solver(i).sdp;
solver(i).lp      = 1;
solver(i).qp      = 0;
solver(i).gp      = 0;
solver(i).ncqp    = 0;
solver(i).socp    = 0;
solver(i).rotsocp = 0;
solver(i).qc = 0;
solver(i).equalities = 1;
solver(i).duals   = 0;
solver(i).complex = 0;
solver(i).integer = 0;
solver(i).binary  = 0;
solver(i).parametric = 0;
solver(i).checkfor= {'callnone.m'};
solver(i).call    = 'callnone';
solver(i).subcall = '';
solver(i).show    = 0;
i = i+1;
